package csnetwork;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.text.DefaultCaret;

import java.net.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.Color;
import java.io.*;
import java.util.Scanner;

//수신 클래스 
//서버의 메시지를 받는 역할 <스레드>
class ClientReceiver extends Thread{
	  Socket socket;
	  ObjectInputStream in;
	  ClientGUI gui;
	  
	  // (서버와 수신하는) 스레드 객체의 멤버 초기화 
	  ClientReceiver(ClientGUI gui, Socket socket) {
		  this.gui=gui;
		  this.socket = socket;
		  try {
		    in = new ObjectInputStream(socket.getInputStream());  // 데이터 수신을 편하게 하도록 도와주는 객체 (서버 -> 클라이언트)
		  } 
		  catch(Exception e) {
			  System.out.println("예외 메시지:"+e.getMessage());
		  }
	  }
	  
	  @Override
	  public void run() {
		  // 반복 (수신 대기하다가 메시지 받으면 저장 -> TextArea에 출력) 
		  while(in!=null) {
			  try {
				  Packet packet = (Packet)in.readObject();
			      String msg=packet.get_msg();

			     // 멤버 리스트
			     if(packet.get_cmd()==0x40) {
			    	 gui.txtarea2.append(msg);
			    	 gui.stt_field.setText("0x40");
			     }
			     // 처음 온 메시지
			     else if(packet.get_cmd()==0x20) {
			    	 gui.txtarea.append(msg+'\n');
			    	 gui.stt_field.setText("0x20");
			     }
			     // 메시지  
			     else if(packet.get_cmd()==0x21) {
			    	 gui.txtarea.append(msg+'\n');
			    	 gui.stt_field.setText("0x21");
			     }
			     // 음식점
			     else if(packet.get_cmd()==0x30){
			    	 gui.txtarea.append(msg+'\n');
			    	 gui.stt_field.setText("0x30");
			     }
			   }
			  catch(Exception e) {
				  System.out.println("예외 메시지:"+e.getMessage());
			  }
		  }
	  } 
}

//송신 클래스 
//서버에게 메시지를 보내는 역할 
/*
 * 0x10: 종료
 * 0x20: 처음 통신
 * 0x21: 통신
 * 0x30: 검색
 * 0x40: 인원 확인 
 * */
class ClientSender{
	 Socket socket;
	 ObjectOutputStream out;
	 String name;
	 String code;
	 ClientGUI gui;
	 
	  // 송신 준비 
	  ClientSender(ClientGUI gui, Socket socket, String name, String code) {
		  this.gui=gui;
		  this.socket = socket;
		  try {
		   this.out = new ObjectOutputStream(socket.getOutputStream()); // 데이터 송신을 편하게 하도록 도와주는 객체 (클라이언트 -> 서버) 
		   this.name = name;
		   this.code=code;
		  } 
		  catch(Exception e) {
			  System.out.println("예외 메시지:"+e.getMessage());
		  }
	  }
	  
	  // 처음에는 서버에 이름과 학번 전송
	  public void first() {
		  try {
			   Packet packet=new Packet(0x20,name.concat(code));
			   out.writeObject(packet);
		  }
		  catch(IOException e) {
			  System.out.println("예외 메시지:"+e.getMessage());
		  }
	  } 
	  
	  // 음식점 검색, 메시지 전송 
	  public void msg_send(String msg) {
		  try {
			  	// 음식점 검색
			  	if(msg.charAt(0)=='!')
			  		out.writeObject(new Packet(0x30,msg));
			  	// 메시지 전송 
			  	else
			  		out.writeObject(new Packet(0x21, "["+name+"] "+msg));
		  }
		  catch(IOException e) {
			  System.out.println("예외 메시지:"+e.getMessage());
		  }
	  } 
	  
	  // 멤버 리스트 요청
	  public void request_member() {
		  try {
			  out.writeObject(new Packet(0x40,""));
		  }
		  catch(IOException e) {
			  System.out.println("예외 메시지:"+e.getMessage());
		  }
	  }
}


// 버튼 이벤트 처리
// (서버 연결, 메시지 전송, 인원 수 확인) 
class Action implements ActionListener{
	ClientGUI gui;
	ClientSender sender;
	Thread receiver;
	
	// 생성자
	public Action(ClientGUI gui) {
		this.gui=gui;
		
		// 키 이벤트 생성 <Connect, Send>
		// <Connect>
		gui.name_field.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if(e.getKeyCode()==e.VK_ENTER) {
					connect_func(); // 서버 연결 
					gui.msg_field.requestFocusInWindow(); // 커서 위치 변경
				}
			}
		});
		// <Send>
		gui.msg_field.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				if(e.getKeyCode()==e.VK_ENTER) {
					send_func(); // 메시지 전송
				}
			}
		});
	}
	
	// 학생 검사
	/*
	 * 1. 학번에는 숫자만 포함돼있음 
	 * 2. 학번의 자릿수는 9자리
	 * 3. 학번의 앞 네 자리는 1855~2022 사이 
	 */
	boolean check_code(String code) {
		try {
			int code_int=Integer.parseInt(code);
			int four=Integer.parseInt(code.substring(0,4));
			if(code.length()==9 && four>=1855 && four<=2022)
				return true;
			else
				return false;
		}
		catch(NumberFormatException e) {
			return false;
		}
	}
	
	// 서버 연결 
	public void connect_func() {
	    	// TextArea 초기화
	    	gui.txtarea.setText(null);
	    	gui.txtarea2.setText(null);
	    	
	    	String ip="125.180.136.226";
	    	String port="7777";
	    	
	    	// 컴포넌트 값 가져오기 
	    	String code=gui.code_field.getText();
	    	String name= gui.name_field.getText();
	    	
	    	// 올바른 학번인지 검사
	    	boolean student=check_code(code);
	    	
	    	// 학생이 확인되고 닉네임이 존재하면 서버에 연결
	    	if(student && !name.equals("")) {
	    		try {
	  			    // port 정수화
	  			    int port_int=Integer.parseInt(port);
	  			  
	  			  	// 소켓 생성 
	  			  	Socket socket = new Socket(ip, port_int); 
	        		
					// 서버와 송신하는 객체 생성
					sender = new ClientSender(gui,socket, name, code);
					sender.first();
					
					// 서버와 수신하는 객체 생성 
					receiver = new Thread(new ClientReceiver(gui,socket));
					receiver.start();
					
					// 접속 버튼 비활성화
					gui.cnxt_btn.setEnabled(false);
	  		  }
			  catch(ConnectException err) {
				  // 서버에 접속 실패 
				  gui.txtarea.append("서버와 연결을 실패했습니다.");
				  err.printStackTrace();
			  }
	  		  catch(Exception err) {
				  // 서버에 접속 실패 
				  gui.txtarea.append("서버와 연결을 실패했습니다.");
				  err.printStackTrace();
	  		  }	
	    	}
	    	
	    	// 학생이 아님
	    	else {
				  gui.txtarea.append("올바르지 않은 학번이거나, 닉네임과 학번이 입력되지 않았습니다.");
	    	}
	}
	
	// 전송
	public void send_func() {
    	// 컴포넌트 값 가져오기 
    	String msg=gui.msg_field.getText();
    	
    	// msg가 채워져 있으면 실행 
    	if(!msg.equals("")) {
    		// 메시지 전송
    		try {
        		sender.msg_send(msg);
    		}
    		catch(Exception err){}
    		
    		// 메시지필드 초기화
    		gui.msg_field.setText(null);	
    	}
	}
	
	// 버튼이 눌린 경우 처리 
	@Override
	public void actionPerformed(ActionEvent e) {
		 JButton act=(JButton) e.getSource(); //활동하는 버튼 
	     String actname=act.getText(); // 활동하는 버튼의 텍스트
	            
	        switch(actname) {
	            case "Connect":
	            	connect_func();
	                break;
	                
	            case "Send":
	            	send_func();
	                break;
	           
	            case "Clear Chat":
	            	// TextArea 초기화
	            	gui.txtarea.setText(null);
	                break;
	                
	            case "Refresh":
	            	gui.txtarea2.setText(null);
	            	sender.request_member();
	                break;
	            
	        }
	}
	
}

class ClientGUI extends JFrame{
	// 컴포넌트 선언 
	JLabel code_label;
	JTextField code_field;
	JLabel name_label;
	JTextField name_field;
	JLabel chat_label;
	JButton clear_btn;
	JLabel stt_label;
	JTextField stt_field;
	JTextArea txtarea;
	JScrollPane txtareaplus;
	JLabel msg_label;
	JTextField msg_field;
	JButton cnxt_btn;
	JLabel user_label;
	JButton refresh_btn;
	JTextArea txtarea2;
	JScrollPane txtareaplus2;
	JButton send_btn;
	
	public ClientGUI() {
		// GUI 기본 세팅 
		setSize(670,420);
        setLocation(700,300);
		setTitle("CUK Chat Community");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		/*
		 * 채팅 앱 디자인
		 */
		
		// contentPane 정의 
		JPanel contentPane = new JPanel();
		setContentPane(contentPane);
		contentPane.setLayout(null);
		
		// 패널 정의 
		JPanel panel_0 = new JPanel();
		panel_0.setBounds(0, 0, 470, 400);
		panel_0.setLayout(null);
		add(panel_0);
		
		JPanel panel_1 = new JPanel();
		panel_1.setBounds(470, 0, 200, 400);
		panel_1.setLayout(null);
		add(panel_1);
		
		
		// 패널 0 디자인
		
		// 패널 0 - 1행 		
		code_label=new JLabel("Code:");
		code_label.setBounds(10,10,80,25);
		panel_0.add(code_label);
		
		code_field=new JTextField();
		code_field.setBounds(60,10,160,25);
		panel_0.add(code_field);
		
		name_label=new JLabel("Name:");
		name_label.setBounds(240,10,80,25);
		panel_0.add(name_label);
		
		name_field=new JTextField();
		name_field.setBounds(300,10,160,25);
		name_field.setText(null);
		panel_0.add(name_field);

		// 패널 0 - 2행 
		chat_label=new JLabel("Chat:");
		chat_label.setBounds(10,45,80,25);
		panel_0.add(chat_label);
		
		clear_btn=new JButton("Clear Chat");
		clear_btn.setBounds(60,45,160,25);
		panel_0.add(clear_btn);
		
		stt_label=new JLabel("Status:");
		stt_label.setBounds(240,45,80,25);
		panel_0.add(stt_label);
		
		stt_field=new JTextField();
		stt_field.setBounds(300,45,160,25);
		stt_field.setForeground(new Color(0,0,255));
		stt_field.setText("None");
		panel_0.add(stt_field);
		
		// 패널 0 - 3행 
		txtarea = new JTextArea();
		DefaultCaret caret = (DefaultCaret)txtarea.getCaret(); // auto-scroll 
		caret.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);
		txtareaplus = new JScrollPane(txtarea);
		txtarea.setEditable(false);
		txtareaplus.setBounds(10,80,450,260);
		panel_0.add(txtareaplus);
		
		// 패널 0 - 4행 
		msg_label=new JLabel("Msg:");
		msg_label.setBounds(10,350,80,25);
		panel_0.add(msg_label);
		
		msg_field=new JTextField();
		msg_field.setBounds(60,350,400,25);
		panel_0.add(msg_field);
		
		
		// 패널 1 디자인
		
		// 패널 1 - 1행
		cnxt_btn=new JButton("Connect");
		cnxt_btn.setBounds(0,10,175,25);
		panel_1.add(cnxt_btn);
		
		// 패널 1 - 2행 
		user_label=new JLabel("Users:");
		user_label.setBounds(0,45,80,25);
		panel_1.add(user_label);
		
		refresh_btn=new JButton("Refresh");
		refresh_btn.setBounds(50,45,125,25);
		panel_1.add(refresh_btn);
		
		// 패널 1 - 3행 
		txtarea2 = new JTextArea();
		DefaultCaret caret2 = (DefaultCaret)txtarea2.getCaret(); // auto-scroll 
		caret2.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);
		txtareaplus2 = new JScrollPane(txtarea2);
		txtarea2.setEditable(false);
		txtareaplus2.setBounds(0,80,175,260);
		panel_1.add(txtareaplus2);
		
		// 패널 1 - 4행 
		send_btn=new JButton("Send");
		send_btn.setBounds(0,350,175,25);
		panel_1.add(send_btn);
		
		
		// 버튼 이벤트 처리
		Action action=new Action(this);
		clear_btn.addActionListener(action);
		cnxt_btn.addActionListener(action);
		refresh_btn.addActionListener(action);
		send_btn.addActionListener(action);
		
		// 시각화 
        setVisible(true);
	}
}

public class client{
	
	public static void main(String[] args) {
		ClientGUI gui=new ClientGUI();
	}

}
